/*
  File: unixc.c
  Description: Fortran/C interface routines for CLI
  Author: A.Shirahashi
  Date: 22-Jun-1992

  Public: opendir_, readdir_, closedir_, fisatty_, fdelete_, fsystem_
          fredirect_, frestore_, fresume_
*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>


/* output redirection routines */

static char *path = (char *)NULL;

int fredirect_(int *fh, char *file,int length)
{
  int fs, ff;
  FILE *fp;

  fprintf(stderr,"redirect: *fh=%d, file=<%.*s>, length=%d\n",*fh,length,file,length);
  if( *file != ' ' ) {
    fprintf(stderr,"*file != SPACE\n");
    if( path != (char *)NULL ) free(path);
    path = (char *)malloc(length+1);
    strncpy(path,file,length);
    *(path+length) = '\0';
  } else if( path == (char *)NULL ) {
    return(-1);
  }

/*
  if( (fp = fopen(path,"w")) == (FILE *)NULL ) {
    free(path);
    return(-1);
  }
  ff = fileno(fp);
*/
  if( path == (char *)NULL ) {
     printf(stderr,"path = NULL\n");
     return(-1);
  }
  if( *file != ' ' ) {
    fprintf(stderr,"try to open %s (create)\n",path);
    ff = open(path,O_WRONLY|O_CREAT|O_TRUNC,0666);
    fprintf(stderr,"try to open %s (create) after open\n",path);
  } else {
    fprintf(stderr,"try to open %s (append)\n",path);
    ff = open(path,O_WRONLY|O_APPEND,0);
    fprintf(stderr,"try to open %s (append) after open\n",path);
  }
  if( ff < 0 ) {
    fprintf(stderr,"can't open file %s\n",path);
    free(path);
    path = (char *)NULL;
    return(-1);
  }
  fs = dup(*fh);
  dup2(ff,*fh);
  close(ff);
  free(path);
  return(fs);
}

void frestore_(int *fh,int *fs)
{
  /*fprintf(stderr,"frestore: *fh=%d,*fs=%d\n",*fh,*fs);*/
  if( *fs > 0 ) {
    close(*fh);
    dup2(*fs,*fh);
    close(*fs);
  }
}

void freclose_(int *fh,int *fs)
{
  /*fprintf(stderr,"freclose: *fh=%d,*fs=%d\n",*fh,*fs);*/
  if( *fs > 0 ) {
    close(*fh);
    dup2(*fs,*fh);
    close(*fs);
    if( path != (char *)NULL ) {
      free(path);
      path = (char *)NULL;
    }
  }
}
