/*
  File: unixc.c
  Description: Fortran/C interface routines for CLI
  Author: A.Shirahashi
  Date: 22-Jun-1992

  History: 6-Sep-1993, patch for OSF/alpha
           1-Apr-1994, add fsignal_ (by, R.Itoh)
          19-Nov-1994, add fexecp_

  Public: opendir_, readdir_, closedir_, fisatty_, fdelete_, fsystem_
          fredirect_, frestore_, fflush_, fchdir_, fgetwd_, fsignal_
*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>

#define MAXFP 8

static FILE *filep[MAXFP] = { 0,0,0,0,0,0,0,0 };
static int fpp;

int opendir_(char *filename,int length)
{
  char *p;
  FILE *fp;

  p = (char *)malloc(length+9);
  strcpy(p,"/bin/ls ");
  strncat(p,filename,length);
  fp = popen(p,"r");
  free(p);
  for( fpp = 1; fpp < MAXFP; fpp++ ) {
	 if( filep[fpp] == 0 ) {
		 filep[fpp] = fp;
		 return(fpp);
	 }
  }
  return(0);
}

int readdir_(int *dirp,char *filename,int length)
{
  int status;
  char *p;

  if( *dirp == 0 || filep[*dirp] == 0 ) return(0);
  status = (int)fgets(filename,length,filep[*dirp]);
  if( status != NULL ) {
    if( (p = strchr(filename,'\n')) != (char *)NULL ) {
      while( p < filename+length ) {
        *p++ = ' ';
      }
    }
  }
  return(status);
}

void closedir_(int *dirp)
{
  if( *dirp == 0 || filep[*dirp] == 0 ) return;
  pclose(filep[*dirp]);
  filep[*dirp] = 0;
}


int fisatty_(int *dp)
{
  return( isatty(*dp) );
}


void fdelete_(char *file, int length)
{
  char *p = (char *)malloc(length+1);

  strncpy(p,file,length);
  *(p+length) = '\0';
#ifdef DEBUG
  fprintf(stderr,"fdelete_:file=[%s]\n",p);
#endif
  unlink(p);
  free(p);
}

void fsystem_(char *comand, int lcmd)
{
  if( lcmd > 0 ) {
    char *p = (char *)malloc(lcmd+1);
    strncpy(p,comand,lcmd);
    *(p+lcmd) = '\0';
    (void)system(p);
    free(p);
  } else {
    char *p;
    if( (p = (char *)getenv("SHELL")) != (char *)NULL ) {
      (void)system(p);
    } else {
      (void)system("/bin/csh");
    }
  }
}

/* output redirection routines */

int fredirect_(int *fh, char *file,char *mode,int length,int lmode)
{
  int fs, ff;
  char *path;
  FILE *fp;

/*
  fprintf(stderr,"redirect: *fh=%d, file=<%.*s>, length=%d\n",*fh,length,file,length);
*/
  path = (char *)malloc(length+1);
  strncpy(path,file,length);
  *(path+length) = '\0';

  if( path == (char *)NULL ) {
     printf(stderr,"path = NULL\n");
     return(-1);
  }

  if( *mode == 'w' ) {
    /*fprintf(stderr,"try to open %s (create)\n",path);*/
    ff = open(path,O_WRONLY|O_CREAT|O_TRUNC,0666);
    free(path);
  } else if( *mode == 'a' ) {
    /*fprintf(stderr,"try to open %s (append)\n",path);*/
    ff = open(path,O_WRONLY|O_APPEND,0);
    free(path);
  } else {
    fprintf(stderr,"fredirect: unknown mode (%c)\n",*mode);
    free(path);
    return(-1);
  }
  if( ff < 0 ) {
    fprintf(stderr,"can't open file %s\n",path);
    return(-1);
  }
  fs = dup(*fh);
  /*close(*fh);*/
  dup2(ff,*fh);
  close(ff);
  return(fs);
}

void frestore_(int *fh,int *fs)
{
  /*fprintf(stderr,"frestore: *fh=%d,*fs=%d\n",*fh,*fs);*/
  if( *fs > 0 ) {
    close(*fh);
    dup2(*fs,*fh);
    close(*fs);
  }
}

void fflush_(int *fh)
{
  if( *fh == 1 ) {
    fflush(stdout);
  }
}

void fchdir_(char *path,int length)
{
  char *p = (char *)malloc(length+1);

  strncpy(p,path,length);
  *(p+length) = '\0';
  (void)chdir(p);
  free(p);
}

void fgetwd_(char *path,int length)
{
  int len;  
  char *p;
#ifdef _HPUX_SOURCE
  (void)getcwd(path,length);
#else
  (void)getwd(path);
#endif
  for( p = path + strlen(path); p < path + length; ++p ) {
    *p = ' ';
  }
}

  
int fsignal_(signum,funct,flag)
      int  *signum, *flag;
      int  *funct;
{
      int  signo, istat;
      int  handler;
      void *oldhand;


      signo = *signum;

      if (*flag < 0)          handler = (int)funct;
        else if (*flag == 0)  handler = (int)SIG_DFL;
        else if (*flag == 1)  handler = (int)SIG_IGN;
        else                  handler = *flag;

      oldhand = signal(signo,handler);
      istat   = (int)oldhand;
      if (istat >= 0)        return istat;
      return -errno;
}

void fexecp_(char *inbuf,char *result,int *reslen,
             int inlen,int outlen)
{
  char *p;
  FILE *fp;
  int n;

  p = (char *)malloc(inlen+1);
  strncpy(p,inbuf,inlen);
  *(p+inlen) = '\0';
  /*printf("inbuf=%s\n",p);*/
  /*printf("inlen=%d\n",inlen);*/
  fp = popen(p,"r");
  free(p);
  n = outlen; 
  (void)fgets(result,n,fp);
  (void)pclose(fp);
  n = strlen(result);
  *(result+n) = ' ';
  if( *(result+n-1) == '\n' ) {
    n--;
    *(result+n) = ' ';
  }
  *reslen = n;
}


