C Subroutine to calculate theory or model
C
C
      Subroutine min_funct(x,xwidth,par,ichoice,all,bak,sig)
C----------------------------------------------------------------------
C-
C-   Purpose and Methods : Function to be used in DISPLAY-MINUIT
C-            by Sunyaev-Titarchuk comptonization
C-   Inputs  : par(1)->intensity at 1keV
C-             par(2)->plasma temperature
C-             par(3)->optical depth
C-             par(4)->normalize energy
C-   Outputs :
C-   Controls:
C----------------------------------------------------------------------
      Implicit None
! input:
      Real*8       x
      Real*8       xwidth             !
      Real*8       par(*)             ! Par(2) =
      Integer      ichoice
C output:
      Real*8       all
      Real*8       bak
      Real*8       sig
C local:
      Real*8    intensity,temperature,depth,norm_e
      Real*8 flux
C
      all = 0.
      bak = 0.
      sig= 0.
      intensity=par(1)
      temperature=par(2)
      depth=par(3)
      norm_e=par(4)
C
      Call sun_tit_23a(x,temperature,depth,intensity,norm_e,flux)
      sig = flux
      all = bak+sig
      Return
      End
      Subroutine sun_tit_23A(e_kev,temperature,depth,intensity,e_norm,
     &  flux)
C----------------------------------------------------------------------
C-
C-   Purpose and Methods : calc sekibun term in Sunyaev-Titarchuk
C-                         A&A 86,p.121 (23A)
C-   Inputs  : e_kev,temperature,intensity
C-   Modified:
C-   Outputs : flux
C-
C-   Created  25-AUG-1992   N.Yamaoka
C-
C----------------------------------------------------------------------
      Implicit None
C----------------------------------------------------------------------
c input:
      Real*8 e_kev         !Energy of gamma -ray in KeV
      Real*8 temperature   !plasma temperature
      Real*8 depth         !optical depth
      Real*8 intensity     !intensity=1 at norm_e
      Real*8 e_norm
c output:
      Real*8 flux          !photon/cm**2/sec/keV
c local
      Real*8 x,alpha,gamma,val,gamma_val
      Real*8 kum_1,kum_2,fac_1,fac_2
      Real*8 flux_norm,x_norm
      Integer ifail
c const
      Real*8 PI,ELE
      Parameter(PI=3.141592653d0,ELE=510.999d0)
c function
      Real*8 s14aaf        !NAG gamma function
      x=e_kev/temperature
      x_norm=e_norm/temperature
      ifail=0
      gamma = PI*PI*ELE/3.0/(depth+2.0/3.0)/(depth+2.0/3.0)/temperature
      alpha = sqrt(2.25+gamma)-1.5
c      Write(*,*) gamma,alpha
c calc factors for Kummer value
      fac_1=s14aaf(-2.0d0*alpha-3.0d0,ifail)/s14aaf(-alpha-3.0d0,ifail)
      fac_2=s14aaf( 2.0d0*alpha+3.0d0,ifail)/s14aaf(alpha,ifail)
c calc 1keV flux at normalization
      Call kummer(x_norm,alpha     ,2.0*alpha+4.0,kum_1)
      Call kummer(x_norm,-alpha-3.0,-2.0*alpha-2.0,kum_2)
      flux_norm=
     &   exp(-x_norm)*real(kum_1*(x_norm**( alpha+2.0))*fac_1
     &                    +kum_2*(x_norm**(-alpha-1.0))*fac_2)
c calc E kev photon flux
      Call kummer(x,alpha     ,2.0*alpha+4.0,kum_1)
      Call kummer(x,-alpha-3.0,-2.0*alpha-2.0,kum_2)
      flux=exp(-x)*real(kum_1*(x**( alpha+2.0))*fac_1
     &                 +kum_2*(x**(-alpha-1.0))*fac_2)
      flux= flux/flux_norm*intensity
  999 Return
      End
c----------------
c kummer
c---------------
      Subroutine Kummer(x,a, c,val )
c Kummer's polynomial :conf hypergeometric function
      Implicit None
c input
      Real*8 x        !X
      Real*8 a,c      !parameter
c output:
      Real*8 val
c local
      Integer i
      Real*8 m, factor
c const
      Real*8 BIG,SMALL  !for VMS
      Parameter (BIG=1d30,SMALL=1d-20)
      m = 1.0D0
      val = 1.0D0
c      write(*,*) 'X=',x
      Do i = 1, 9999
        factor = (a+dble(i)-1.0d0)*x/((c+dble(i)-1.0d0)*dble(i))
        If(abs(factor).ge.BIG) Then
c           write(*,*) 'Factor become BIG i=',i
          Goto 999
        Endif
        m=m*factor
        If(abs(m).le.SMALL) Then
c           Write(*,*) 'M   become SMALL i=',i
          Goto 999
        Endif
        val=val+m
      Enddo
  999 Continue
      End


