C
C File: dis45tool2.f
C Date: 18-Sep-1992
C Desription: display45_tool.f -> dis45tool1.f, dis45tool2.f
C
C History:
C   18-Jan-93, add DIS45_hrebinx
C    3-Feb-93, add DIS45_redirect, DIS45_reclose
C   14-Oct-93, DIS45_rebin1: check the validity of arguments
C   12-Mar-94, cmdline: 80 -> 256
C
C $Revision: 1.2 $
C $Date: 1994/03/12 06:31:15 $ 
C $Author: sirahasi $
C
C *****
C DIS45_hrebin1
C DIS45_hrebin2
C DIS45_chan2limit1
C DIS45_chan2limit2
C DIS45_getchan
C DIS45_chan2limitsn
C DIS45_HixC
C DIS45_HijxyC
C DIS45_getxbins
C DIS45_hinfo0
C DIS45_hinfo1
C DIS45_hinfo2
C *****
C
      Subroutine DIS45_hrebin1( id,nx,xfirst,xlast,yarray,earray,error )
C
C Similar to HBOOK Hrebin, but does not return X and DX
C
      Implicit None
C input
      Integer id, nx, xfirst, xlast
C output
      Real  yarray(nx), earray(nx)
      Integer  error
C local
      Integer  i, isx, idx, xstep
C function
      Real  Hi, Hie
C begin
      If( nx.le.0 ) Then
        Return
      End If
      xstep = (xlast - xfirst + 1)/nx
      isx = xfirst
      Do idx = 1, nx
        yarray(idx) = 0.0
        earray(idx) = 0.0
        Do i = isx, isx + xstep - 1
          yarray(idx) = yarray(idx) + Hi(id,i)
          earray(idx) = earray(idx) + Hie(id,i)**2
        End Do
        earray(idx) = Sqrt(earray(idx))
        isx = isx + xstep        
      End Do
      Return
      End
C
C
      Subroutine DIS45_hrebinX( id,nx,xfirst,xlast,
     &                          xarray,yarray,earray,error )
C
C Similar to dis45_rebin1, but returns X,Y,E
C
      Implicit None
C input
      Integer id, nx, xfirst, xlast
C output
      Real  xarray(*), yarray(*), earray(*)
      Integer  error
C local
      Integer  i, isx, idx, xstep
C function
      Real  Hi, Hie
C begin
      xstep = (xlast - xfirst + 1)/nx
      isx = xfirst
      Do idx = 1, nx
        yarray(idx) = 0.0
        earray(idx) = 0.0
        Call Hix( id,isx,xarray(idx) )
        Do i = isx, isx + xstep - 1
          yarray(idx) = yarray(idx) + Hi(id,i)
          earray(idx) = earray(idx) + Hie(id,i)**2
        End Do
        earray(idx) = Sqrt(earray(idx))
        isx = isx + xstep        
      End Do
      Call Hix( id, isx, xarray(nx+1) )
      Return
      End
C
C
      Subroutine DIS45_hrebin2( id,nx,xfirst,xlast,ny,yfirst,ylast,
     &                          array,error )
C
C 2 dimensional version of HBOOK Hrebin
C
      Implicit None
C input
      Integer  id, nx, ny, xfirst, yfirst, xlast, ylast
C output
      Real  array(nx,ny)
      Integer  error
C local
      Integer  i, j, isx, jsy, idx, jdy, xstep, ystep
C function
      Real  Hij
C begin
      xstep = (xlast - xfirst + 1)/nx
      ystep = (ylast - yfirst + 1)/ny
C
      jsy = yfirst
      Do jdy = 1, ny
        isx = xfirst 
        Do idx = 1, nx
          array(idx,jdy) = 0.0
          Do j = jsy, jsy + ystep - 1
            Do i = isx, isx + xstep -  1
              array(idx,jdy) = array(idx,jdy) + Hij(id,i,j)
            End Do
          End Do
          isx = isx + xstep
        End Do
        jsy = jsy + ystep
      End Do
C
      error = 0
      Return
      End
C
C
      Integer Function DIS45_getlimit1( id,nx,ifirst,ilast,error )
C
      Implicit None
C input
      Integer  id
C output
      Integer  nx, ifirst, ilast, error
C local
      Character * 80  title
      Real  xmin, xmax, bin, low, high
C begin
      Call DIS45_hinfo1(id,title,nx,xmin,xmax)
C
      Call DIS45_getchan('from',nx,xmin,xmax,xmin,low,ifirst)
      Call DIS45_getchan('to',nx,low,xmax,xmax,high,ilast)
C
      bin = (xmax - xmin)/nx
C
      If( ifirst.eq.0 ) Then
        Call Hxi(id,low+bin*0.01,ifirst)
        ifirst = Max( 1, ifirst )
C       ifirst = Max( 1, Int( (low - xmin)/bin + 1.01 ) )
      End If
      If( ilast.eq.0 ) Then
        Call Hxi(id,high-bin*0.01,ilast)
        ilast = Min( nx, ilast )
C       ilast = Min( nx, Int( (high - xmin)/bin + 0.99 ) )
      End If
C
      nx = ilast - ifirst + 1
C
      If( xmin.ge.xmax .or. ifirst.gt.ilast ) Then
        error = 20 ! you have rejected everything
        DIS45_getlimit1 = 0
      Else
        DIS45_getlimit1 = 1
      End If
C      
      Return
      End
C
C
      Integer Function DIS45_getlimit2( id,dir,
     &                                  nx,ifirst,ilast,error )
C
      Implicit None
C input
      Integer  id
      Character * 1  dir
C output
      Integer  nx, ifirst, ilast, error
C local
      Character * 80  title
      Integer  ny
      Real  xmin, xmax, ymin, ymax, bin, low, high
C begin
      If( dir.eq.'X' ) Then
        Call DIS45_hinfo2(id,title,nx,xmin,xmax,ny,ymin,ymax)
      Else If( dir.eq.'Y' ) Then
        Call DIS45_hinfo2(id,title,ny,ymin,ymax,nx,xmin,xmax)
      Else
        Write(*,*) 'DIS45_getlimit2: unknown dir: ', dir
      End If
C
      Call DIS45_getchan(dir//' from',nx,xmin,xmax,xmin,low,ifirst)
      Call DIS45_getchan(dir//' to',nx,low,xmax,xmax,high,ilast)
C
      bin = (xmax - xmin)/nx
C
      If( ifirst.eq.0 ) Then
        If( dir.eq.'X' ) Then
          Call Hxyij(id,low+bin*0.01,ymin,ifirst,ny)
        Else If( dir.eq.'Y' ) Then
          Call Hxyij(id,ymin,low+bin*0.01,ny,ifirst)
        End If
        ifirst = Max( 1, ifirst )
C       ifirst = Max( 1, Int( (low - xmin)/bin + 1.01 ) )
      End If
      If( ilast.eq.0 ) Then
        If( dir.eq.'X' ) Then
          Call Hxyij(id,high-bin*0.01,ymin,ilast,ny)
        Else If( dir.eq.'Y' ) Then
          Call Hxyij(id,ymin,high-bin*0.01,ny,ilast)
        End If
        ilast = Min( nx, ilast )
C       ilast = Min( nx, Int( (high - xmin)/bin + 0.99 ) )
      End If
C
      nx = ilast - ifirst + 1
C
      If( xmin.ge.xmax .or. ifirst.gt.ilast ) Then
        error = 20 ! you have rejected everything
        DIS45_getlimit2 = 0
      Else
        DIS45_getlimit2 = 1
      End If
C      
      Return
      End
C
C
      Subroutine DIS45_chan2limit1( id,ifirst,ilast,low,high )
C
      Implicit None 
C input
      Integer  id, ifirst, ilast
C output
      Real  low, high
C begin
      Call Hix(id,ifirst,low)
      Call Hix(id,ilast+1,high)
      Return
      End
C
C
      Subroutine DIS45_chan2limit2( id,dir,ifirst,ilast,low,high )
C
      Implicit None 
C input
      Integer id
      Character * 1  dir
      Integer  ifirst, ilast
C output
      Real  low, high
C local
      Real  x, y
C begin
      If( dir.eq.'X' ) Then
        Call Hijxy(id,ifirst,1,low,y)
        Call Hijxy(id,ilast+1,1,high,y)
      Else If( dir.eq.'Y' ) Then
        Call Hijxy(id,1,ifirst,x,low)
        Call Hijxy(id,1,ilast+1,x,high)
      Else
        Write(*,*) 'DIS45_chan2limit2: unknown dir: ', dir
      End If
C
      Return
      End
C
C
      Subroutine DIS45_getchan( prompt,nbin,minedge,maxedge,default,
     &                          value,channel )
C
      Implicit None 
C input
      Character * (*)  prompt
      Integer  nbin
      Real  minedge, maxedge, default
C output
      Real  value
      Integer  channel
C local
      Character * 16  str
      Integer  ntloc, idh, icx, icy, istat
      Real  xloc, yloc
C begin
  100 Continue 
      value = default
      Call FltrdX( prompt,value,str )
C
      If( str(1:1) .eq. '?' ) Then
        Call HPloc(-1,ntloc,xloc,yloc,idh,icx,icy,istat)
        If( ntloc.lt.10 ) Goto 100
        If( Index(prompt,'Y').gt.0 .or. Index(prompt,'y').gt.0 ) Then
          channel = icy
          value = yloc
        Else
          channel = icx
          value = xloc
        End If
      Else If( str(1:1) .eq. '#' ) Then
C       Call CLatoi( str(2:len),channel )
        channel = Nint(value)
        If( channel.lt.0 ) Then
          channel = nbin + channel + 1
        End If
        If( channel.le.0 .or. channel.gt.nbin ) Goto 100
      Else
        channel = 0
C       Call CLatof( str(1:len),value )
        If( value.lt.minedge .or. value.gt.maxedge ) Goto 100
      End If
C
      Return
      End
C
C
      Subroutine DIS45_chan2limits( nbin,minedge,maxedge,ifirst,ilast,
     &                              low,high )
C
      Implicit None
C input
      Integer  nbin, ifirst, ilast
      Real  minedge, maxedge
C output
      Real  low, high
C local
      Real  bin
C begin
      bin = (maxedge - minedge)/nbin
      low = minedge + (ifirst - 1)*bin
      high = minedge + ilast*bin
      Return
      End
C
C
      Subroutine DIS45_HixC(id,i,x)
C ... return the center value of i'th bin
      Implicit None 
C input
      Integer  id, i
C output
      Real  x
C local
      Real  x1, x2
C begin
      Call Hix(id,i,x1)
      Call Hix(id,i+1,x2)
      x = (x1 + x2)/2.0
      Return
      End
C
C
      Subroutine DIS45_HijxyC(id,i,j,x,y)
C ... return the center value of (i,j)'th bin
      Implicit None 
C input
      Integer  id, i, j
C output
      Real  x, y
C local
      Real  x1, x2, y1, y2
C begin
      Call Hijxy(id,i,j,x1,y1)
      Call Hijxy(id,i+1,j+1,x2,y2)
      x = (x1 + x2)/2.0
      y = (y1 + y2)/2.0
      Return
      End
C
C
      Subroutine DIS45_getxbins( id,ifirst,ilast,xbins )
C
      Implicit None 
C input
      Integer  id, ifirst, ilast
C output
      Real  xbins(*)
C local
      Integer  I
C begin
      Do I = ifirst, ilast + 1
        Call Hix(id,I,xbins(I - ifirst + 1))
      End Do
      Return
      End
C
C
      Subroutine DIS45_hinfo0(id,title,dim)
C return title and dimension
      Implicit None
C input
      Integer id
C output
      Character * (*)  title
      Integer  dim
C local
      Integer  nx, ny
      Real  xmin, xmax, ymin, ymax
      Integer  nwt, loc
      Integer  nvar
      Character * 8  tags
C function
      Logical  Hexist
C begin
      If( Hexist(id) ) Then
        Call Hgive(id,title,nx,xmin,xmax,ny,ymin,ymax,nwt,loc)
        If( nx.eq.0 .and. ny.eq.0 ) Then
          dim = 3
          Call DIS45_hinfon(id,title,nvar,1,tags)
        Else If( ny.eq.0 ) Then
          dim = 1
        Else If( ny.gt.0 ) Then
          dim = 2
        Else
          Print *, 'DIS45_hinfo0: can''t happen'
          dim = 0
        End If
      Else
        dim = 0
      End If
      Return
      End
C
C
      Subroutine DIS45_hinfo1(id,title,nx,xmin,xmax)
C
      Implicit None
C input
      Integer id
C output
      Character * (*)  title
      Integer  nx
      Real  xmin, xmax
C local
      Integer  ny, nvar
      Real  ymin, ymax
      Integer  nwt, loc
      Character * 8  tags
C function
      Logical  Hexist
C begin
      If( Hexist(id) ) Then
        Call Hgive(id,title,nx,xmin,xmax,ny,ymin,ymax,nwt,loc)
        If( nx.eq.0 .and. ny.eq.0 ) Then
          Call DIS45_hinfon(id,title,nvar,1,tags)
          Call Hnoent(id,nx)
        End If
      Else
        nx = 0
      End If
      Return
      End
C
C
      Subroutine DIS45_hinfo2(id,title,nx,xmin,xmax,ny,ymin,ymax)
C
      Implicit None
C input
      Integer id
C output
      Character * (*)  title
      Integer  nx, ny
      Real  xmin, xmax, ymin, ymax
C local
      Integer  loc, nwt
C function
      Logical  Hexist
C begin
      If( Hexist(id) ) Then
        Call Hgive(id,title,nx,xmin,xmax,ny,ymin,ymax,nwt,loc)
      Else
        nx = 0
        ny = 0
      End If
      Return
      End
C
C
      Subroutine DIS45_plot2(id,opt,npar,par,error)
C
C common
      Include 'pawc.inc'
C input
      Integer  id, npar
      Real  par(*)
      Character * (*)  opt
C output
      Integer  error
C local
      Character * 80  title
      Integer  nx, ny
      Real  xmin, xmax, ymin, ymax
      Integer  idbuf, loc
C begin
      Call DIS45_hinfo2(id,title,nx,xmin,xmax,ny,xmin,xmax)
      If( ny.eq.0 ) Return
C
      If( DIS45_malloc(nx*ny,idbuf,loc,error).le.0 ) Return
C
      Call Hunpak(id,Rmem(loc),'HIST',1)
      Call IGtabl(nx,ny,Rmem(loc),npar,par,opt)
C
      Call DIS45_free(idbuf)
      Return
      End
C
C
      Subroutine DIS45_redirect(file, lun, error)
C
      Implicit None
C common
      Common /dis45pipe/ cmdline
      Character * 256  cmdline
C input
      Character * (*)  file
C ouput
      Integer  lun
      Integer  error
C local
      Integer  status, L
C function
      Integer  Lenrd
C begin
      cmdline = ' '
      Call DIS45_getlun(lun)
C
      L = Lenrd(file)
C
      If( file(1:2).eq.'>>' ) Then
        Call CLopen(file(3:),lun,'A',status,' ')
      Else If( file(1:1).eq.'>' ) Then
        Call CLopen(file(2:),lun,'W',status,' ')
      Else If( file(1:1).eq.'|' ) Then
        cmdline = file(2:)
        Call CLopen('display45.pip',lun,'W',status,' ')
      Else If( file(L:L).eq.'|' ) Then
         cmdline(:L) = file(:L)
         Call CLexec(cmdline(:L-1),' ','display45.pip')
         Call CLopen('display45.pip',lun,'R',status,' ')
      Else
        Call DIS45_freelun(lun)
        error = 23 ! illegal option
        Return
      End If
C
      If( status.ne.0 ) Then
        error = 25 ! can't open file for output
      Else
        error = 0
      End If
      Return
      End
C
C
      Subroutine DIS45_reclose(lun)
C
      Implicit None
C common
      Common /dis45pipe/ cmdline
      Character * 256  cmdline
C input
      Integer  lun
C local
      Integer  L
C function
      Integer  Lenrd
C begin
      Close( lun )
      Call DIS45_freelun(lun)
C
      L = Lenrd(cmdline)
C
      If( L.gt.0 ) Then
        If( cmdline(L:L).eq.'|' ) Then
          Call CLfdel('display45.pip')
        Else
          Call CLexec(cmdline(:L),'display45.pip',' ')
          Call CLfdel('display45.pip')
        End If
      End If
C
      Return
      End
