C
C File: HLavx.for
C Description: makes average and std. deviation histograms for 2D plot
C Data: 25-May-1992
C Author: A.Shirahashi
C Original: FACOM version
C
C History: 27-May-1992, make use of HBOOKB
C          30-Jun-1992, Fltrd -> FltrdL
C          23-Jul-1992, add HLavy
C           3-Aug-1992, protection for too small number of data points
C          18-Nov-1993, do not use MNstat (due to Itoh's claim)
C
C Public: HLavx, HLavy
C
      Subroutine HLavx( error )
C
      Implicit None
C common
      Include 'pawc.inc'
C output
      Integer  error
C local
      Integer  here, dim, id, idav, idsg, idwork
      Integer  nx, ny, i, my
      Integer  idbuf, loc, locx, locy
      Integer  locav, locae, locsg, locse
      Integer  rej1, rej2, status
      Real  xmin, xmax, ymin, ymax
      Real  ev, es, dv, ds
      Real  plcut, pucut
      Character * 80  title
      Logical  dogauss
C function
      Integer DIS45_needhist, DIS45_makehist, DIS45_malloc
C begin
C     ...check if histogram is ok
      If( DIS45_needhist(2,error).le.0 ) Return
      Call DIS45_thishist(here, id, dim)
C
      Call DIS45_hinfo2(id,title,nx,xmin,xmax,ny,ymin,ymax)
C
C     ...allocate buffers
      If( DIS45_malloc( ny*3+nx*4+1,idbuf,loc,error ).le.0 ) Return
C
C     ...set up new histograms
      If( DIS45_makehist(id,1,idav,error).le.0 ) Return
      Call Hbook1( idav,'AVX '//title,nx,xmin,xmax,0.0)
      If( DIS45_makehist(id,1,idsg,error).le.0 ) Return
      Call Hbook1( idsg,'SGX '//title,nx,xmin,xmax,0.0)
C                        
C     ...ask about options
      Call Logrd('gaussian fit?',dogauss)
      plcut = 0.0
      pucut = 0.0
      Call FltrdL('cut at lower side?',plcut,0.0,1.0)
      Call FltrdL('cut at upper side?',pucut,0.0,1.0)
      If( plcut+pucut.ge.1.0 ) Then
        error = 19 ! you have rejectted everyting
        Return
      End If
C
      Call DIS45_locate( idbuf, loc )
      locx  = loc   + ny
      locy  = locx  + ny + 1
      locav = locy  + ny
      locae = locav + nx
      locsg = locae + nx
      locse = locsg + nx
C
      rej1 = 0
      rej2 = 0
C
C     ...loop over x bins
      Do i = 1, nx
        Call DIS45_hrebin2(id,1,i,i,ny,1,ny,Rmem(loc),error)
        Call HLav_cut( ny,ymin,ymax,Rmem(loc),plcut,pucut,
     &                 my,Rmem(locx),Rmem(locy) )
        If( my.ge.3 ) Then
          If( DIS45_makehist(id,1,idwork,error).le.0 ) Goto 900
          Call Hbookb(idwork,'AVX_WORK',my,Rmem(locx),0.0)
          Call Hpak(idwork,Rmem(locy))
          Call HLav_cal( idwork,ev,es,dv,ds )
          If( dogauss ) Then
            Call HLav_fit( idwork,ev,es,dv,ds,status )
            If( status.gt.0 ) Then
              Call HLav_rej( rej1,rej2 )
              rej1 = i
            Else
              rej2 = i
            End If
          End If
          Call DIS45_hdelete(idwork)
        Else
          ev = 0.0
          dv = 0.0
          es = 0.0
          ds = 0.0
        End If
        Rmem(locav+i-1) = ev        
        Rmem(locae+i-1) = dv
        Rmem(locsg+i-1) = es        
        Rmem(locse+i-1) = ds        
      End Do        
C
      If( dogauss ) Call HLav_rej( rej1,rej2 )
C
      Call Hpak (idav,Rmem(locav))
      Call Hpake(idav,Rmem(locae))
      Call Hpak (idsg,Rmem(locsg))
      Call Hpake(idsg,Rmem(locse))
C
      Call Hminim(idav,ymin)
      Call Hmaxim(idav,ymax)
C
  900 Continue
      Call DIS45_free(idbuf)
      Return
      End
C
C
      Subroutine HLavy( error )
C
      Implicit None
C common
      Include 'pawc.inc'
C output
      Integer  error
C local
      Integer  here, dim, id, idav, idsg, idwork
      Integer  nx, ny, i, mx
      Integer  idbuf, loc, locx, locy
      Integer  locav, locae, locsg, locse
      Integer  rej1, rej2, status
      Real  xmin, xmax, ymin, ymax
      Real  ev, es, dv, ds
      Real  plcut, pucut
      Character * 80  title
      Logical  dogauss
C function
      Integer DIS45_needhist, DIS45_makehist, DIS45_malloc
C begin
C     ...check if histogram is ok
      If( DIS45_needhist(2,error).le.0 ) Return
      Call DIS45_thishist(here, id, dim)
C
      Call DIS45_hinfo2(id,title,nx,xmin,xmax,ny,ymin,ymax)
C
C     ...allocate buffers
      If( DIS45_malloc( nx*3+ny*4+1,idbuf,loc,error ).le.0 ) Return
C
C     ...set up new histograms
      If( DIS45_makehist(id,1,idav,error).le.0 ) Return
      Call Hbook1( idav,'AVY '//title,ny,ymin,ymax,0.0)
      If( DIS45_makehist(id,1,idsg,error).le.0 ) Return
      Call Hbook1( idsg,'SGY '//title,ny,ymin,ymax,0.0)
C                        
C     ...ask about options
      Call Logrd('gaussian fit?',dogauss)
      plcut = 0.0
      pucut = 0.0
      Call FltrdL('cut at lower side?',plcut,0.0,1.0)
      Call FltrdL('cut at upper side?',pucut,0.0,1.0)
      If( plcut+pucut.ge.1.0 ) Then
        error = 19 ! you have rejectted everyting
        Return
      End If
C
      Call DIS45_locate( idbuf, loc )
      locx  = loc   + nx
      locy  = locx  + nx + 1
      locav = locy  + nx
      locae = locav + ny
      locsg = locae + ny
      locse = locsg + ny
C
      rej1 = 0
      rej2 = 0
C
C     ...loop over y bins
      Do i = 1, ny
        Call DIS45_hrebin2(id,nx,1,nx,1,i,i,Rmem(loc),error)
        Call HLav_cut( nx,xmin,xmax,Rmem(loc),plcut,pucut,
     &                 mx,Rmem(locx),Rmem(locy) )
        If( mx.ge.3 ) Then
          If( DIS45_makehist(id,1,idwork,error).le.0 ) Goto 900
          Call Hbookb(idwork,'AVY_WORK',mx,Rmem(locx),0.0)
          Call Hpak(idwork,Rmem(locy))
          Call HLav_cal( idwork,ev,es,dv,ds )
          If( dogauss ) Then
            Call HLav_fit( idwork,ev,es,dv,ds,status )
            If( status.gt.0 ) Then
              Call HLav_rej( rej1,rej2 )
              rej1 = i
            Else
              rej2 = i
            End If
          End If
          Call DIS45_hdelete(idwork)
        Else
          ev = 0.0
          dv = 0.0
          es = 0.0
          ds = 0.0
        End If
        Rmem(locav+i-1) = ev        
        Rmem(locae+i-1) = dv
        Rmem(locsg+i-1) = es        
        Rmem(locse+i-1) = ds        
      End Do        
C
      If( dogauss ) Call HLav_rej( rej1,rej2 )
C
      Call Hpak (idav,Rmem(locav))
      Call Hpake(idav,Rmem(locae))
      Call Hpak (idsg,Rmem(locsg))
      Call Hpake(idsg,Rmem(locse))
C
      Call Hminim(idav,xmin)
      Call Hmaxim(idav,xmax)
C
  900 Continue
      Call DIS45_free(idbuf)
      Return
      End

      Subroutine HLav_rej( rej1,rej2 )
C
      Implicit None
C
      Integer  rej1, rej2
C begin
      If( rej2.eq.rej1+1 ) Then
        Write(*,'(1H ,A,I3)') 
     &          'gauss fit rejected for bin ', rej2
      Else If( rej2.gt.rej1+1 ) Then
        Write(*,'(1H ,A,I3,A,I3)')
     &          'gauss fit rejected for bins', rej1+1, ' -', rej2
      End If
      Return
      End
C
C
      Subroutine HLav_cut( nx,xmin,xmax,yin,plcut,pucut,
     &                     nout,xout,yout )      
C
      Implicit None
C input
      Integer  nx
      Real  xmin, xmax, yin(nx)
      Real  plcut, pucut
C output
      Integer  nout
      Real  xout(0:nx), yout(nx)
C local
      Integer  i
      Real  xbin, x0
      Real  sum, sum1, sum2, plsum, pusum
C begin
      xbin = (xmax-xmin)/nx
C     ...sum up
      sum = 0.0
      Do i = 1, nx
        sum = sum + yin(i)
      End Do
      If( sum.le.0.0 ) THen
        nout = 0
        Return
      End If
      plsum = sum * plcut
      pusum = sum * (1.0-pucut)
      sum1 = 0.0
      sum2 = 0.0
      nout = 0
C     ...apply cuts
      Do i = 1, nx
        sum2 = sum2 + yin(i)
        If( sum2.gt.plsum .and. sum1.lt.pusum ) Then
          nout = nout + 1                    
          x0 = xmin + (i-1) * xbin
          If( sum1.le.plsum ) Then
            xout(nout-1) = x0 + ((plsum-sum1)/(sum2-sum1)) * xbin
            xout(nout) = x0 + xbin
            yout(nout) = sum2 - plsum
          Else If( sum2.ge.pusum ) Then  
            xout(nout-1) = x0 
            xout(nout)= x0 + ((pusum-sum1)/(sum2-sum1)) * xbin
            yout(nout) = pusum - sum1
          Else If( sum1.gt.plsum .and. sum2.lt.pusum ) Then
            xout(nout-1) = x0
            xout(nout) = x0 + xbin
            yout(nout) = sum2 - sum1     
          Else
            Print *, 'can''t happen in HLavx_cut!'
            Return
          End If
        End If
        sum1 = sum2          
      End Do        
C
      Return
      End
C
C
      Subroutine HLav_cal( id,ev,es,dv,ds )
C
      Implicit None
C input
      Integer  id
C output
      Real  ev, es, dv, ds
C local
      Integer  i
      Real*8  sw, swx, swxx
      Real  x0, x1, x, y
      Character * 80  title
      Integer  nx
      Real  xmin, xmax
C function
      Real  Hi
C     Real  Hstati, Hsum
C begin
      Call DIS45_hinfo1(id,title,nx,xmin,xmax)
C
      sw = 0.0
      swx = 0.0
      swxx = 0.0
C
      Do i = 1, nx
        Call Hix(id,i,x0)
        Call Hix(id,i+1,x1)
        x = (x0 + x1)/2.0
        y = Hi(id,i)
        sw = sw + y
        swx = swx + y * x
        swxx = swxx + y * x * x
      End Do
C
      If( sw.gt.0.0 ) Then 
        ev = swx / sw
        es = Sqrt( Max( swxx/sw - ev**2, 0.0d0 ) )      
        dv = es/Sqrt(sw) 
        ds = dv/Sqrt(2.0)
      Else     
        ev = 0.0
        es = 0.0
        dv = 0.0
        ds = 0.0
      End If
C
C     ...I would like to use the following simple code, but unfortunately
C        Hstati seems not to take care of non-equivalent width bins.
C     ev = Hstati(id,1,'HIST',1)
C     es = Hstati(id,2,'HIST',1)
C     sw = Hsum(id)
C
C     If( sw.gt.0.0 ) Then
C       dv = es/Sqrt(sw)
C       ds = dv/Sqrt(2.0)
C     Else
C       dv = 0.0
C       ds = 0.0
C     End If        
C
      Return
      End
C
C
      Subroutine HLav_fit( id,ev,es,dv,ds,status )
C
      Implicit None
C input
      Integer  id
C output
      Real  ev, es, dv, ds
      Integer  status
C local
      Real  c, av, sd, sig(3), chi2
      Integer  ic                  
      Real  fmin, fedm, errdef
      Integer  npari, nparx, istat
C function
      Real  Hmax
C begin           
      c = Hmax(id)
      av = ev
      sd = es
C
      ic = 0
      Call Hfitga( id,c,av,sd,chi2,ic,sig )
C
C     ...use MINUIT status to check fitting validity 
C     Call MNstat( fmin,fedm,errdef,npari,nparx,istat )  
C     Print *, chi2, fmin, fedm, errdef
C     If( chi2 .lt. 10.0 .and. 
C    &    fedm .lt. errdef*(10.0**(-3)) .and. istat.eq.3 ) Then
C       ev = av
C       es = Abs(sd)
C       dv = sig(2)
C       ds = sig(3)
C       status = 1
C     Else
C       status = 0
C     End If
C
C     ...in stead of the comparison with dv computed by stat. method
      If( sig(2).lt.3.0*dv ) Then
        ev = av
        es = Abs(sd)
        dv = sig(2)
        ds = sig(3)
        status = 1
      Else
        status = 0
      End If
C
      Return
      End
