C+
c File : hlbinoper.f
c Description : Do user-supplied operation on each histogram bin content
c
c Author : Ryosuke Itoh, Physics Division, KEK
c Date : 18 - May - 1993
c        8,9-Jun-1993, some mod. by A.Shirahashi
C        18-Nov-1993, DIS45_dynopen, DIS45_dynclose
C        8-Feb-1994, for 2 dimensional histogram, by A.Shirahashi
C       14-Feb-1994, bug fix: xhigh -> ylow,yhigh
C-
c
      Subroutine HLbinoper ( Error )
c
      Implicit         None
c Arg
      Integer * 4      Error    ! Output
c Common
      Include 'pawc.inc'
c Var
      Integer      here, id, dim
c Function
      Integer  DIS45_needhist
c Main
C     ... require 1 dim or 2 dim
      If ( DIS45_needhist( 0, error ) .le. 0 ) Return
      Call DIS45_thishist ( here, id, dim )
      If ( dim .eq. 1 ) Then
        Call HLbinoper1( error )
      Else If( dim .eq. 2 ) Then
        Call HLbinoper2( error )
      Else 
        error = 30 ! command not valid for N-tuples
        Return
      End If
C
      Return
      End
C
C
      Subroutine HLbinoper1( error )
C
      Implicit None
C output
      Integer  error
c Common
      Include 'pawc.inc'
c Var
      Integer      Id1, Idnew, here, dim
      Integer      Nx, Idbuf, Loc, I
      Real         Xmin, Xmax
      Character * 80   Title
      Integer      Address
      Character * 80   exec_name
      Real         xlow, xhigh, xbin, xwid, y, dy
c
c Function
c
      Integer  DIS45_makehist
      Integer  DIS45_malloc
      Integer  DIS45_dynopen, DIS45_dynlink, DIS45_hasbar
      Real  Hi, Hie
C
C data
      Data  exec_name / 'bin_funct' /
c
c Main
c
c 0. Check current histogram
c
      Call DIS45_thishist( here, id1, dim )
      Call DIS45_hinfo1 ( Id1, Title, Nx, Xmin, Xmax )
c
c
c 1. Obtain Address of User-supplied Function
c
      If( DIS45_dynopen( 'operation function file',exec_name )
     &    .eq.0 ) Then
         error = 9 ! can't open file
         Return
      End If  
C
      address = DIS45_dynlink ( exec_name, 'BIN_FUNCT' )
      If( address .eq. 0 ) Then
        error = 28 ! can't load dynamic link module
        Return
      End If
C     
c 2. Create Working Buffer and resulting histogram
c
      If( DIS45_malloc ( Nx*2, Idbuf, Loc, Error ) .LE. 0 ) Return
      If( DIS45_makehist ( Id1, 1, Idnew, Error) .LE. 0 ) Return
      Call Hcopy( id1,idnew,'BINOPER '//title )
C
c 3. Do operation
c
      Do I = 1, Nx
        Call Hix ( id1, i, xlow )
        Call Hix ( id1, i+1, xhigh )
        xbin = (xlow + xhigh)/2.0
        xwid = xhigh - xlow
        y = Hi( id1,i )
        dy = Hie( id1,i )
        Call HLbinoper_call1
     &         ( %VAL(Address), i, xbin, xwid, y, dy )
        Rmem(loc+i-1) = y
        Rmem(loc+nx+i-1) = dy  
      End Do
C
      Call HPAK ( idnew, Rmem(Loc) )
      If( DIS45_hasbar(id1).ne.0 ) Then
         Call Hpake( idnew, rmem(loc+nx) )
      End If
C
      Call DIS45_fixentry( idnew )
C
C 4. Free working buffer
C
      Call DIS45_free ( Idbuf )
C
C 5. Unlink dynamic load module
C
      Call DIS45_dynclose( address )
C
      Return
      End
C
C
      Subroutine HLbinoper_call1( Func, nbin, xbin, xwid, y, dy )
c
      Implicit         NONE
c
c Arg
      External  Func
      Integer * 4  nbin
      Real * 4  xbin, xwid, y, dy
c begin
      Call Func ( nbin, xbin, xwid, y, dy )
C
      Return
      End
C
C
      Subroutine HLbinoper2( error )
C
      Implicit None
C output
      Integer  error
c Common
      Include 'pawc.inc'
c Var
      Integer      Id2, Idnew, here, dim
      Integer      Nx, ny, Idbuf, Loc, I, j
      Real         Xmin, Xmax, ymin, ymax
      Character * 80   Title
      Integer      Address
      Character * 80   exec_name
      Real         xlow, xhigh, xbin, xwid
      Real         ylow, yhigh, ybin, ywid
      Real         z
c Function
      Integer  DIS45_makehist
      Integer  DIS45_malloc
      Integer  DIS45_dynopen, DIS45_dynlink
      Real  Hij
C data
      Data  exec_name / 'bin_funct2' /
c Main
c
c 0. Check current histogram
c
      Call DIS45_thishist( here, id2, dim )
      Call DIS45_hinfo2 ( Id2, Title, Nx,Xmin,Xmax, ny,ymin,ymax )
c
c 1. Obtain Address of User-supplied Function
c
      If( DIS45_dynopen( 'operation function file',exec_name )
     &    .eq.0 ) Then
         error = 9 ! can't open file
         Return
      End If  
C
      address = DIS45_dynlink ( exec_name, 'BIN_FUNCT2' )
      If( address .eq. 0 ) Then
        error = 28 ! can't load dynamic link module
        Return
      End If
C     
c 2. Create Working Buffer and resulting histogram
c
      If( DIS45_malloc ( nx*ny, Idbuf, Loc, Error ) .LE. 0 ) Return
      If( DIS45_makehist ( Id2, 2, Idnew, Error) .LE. 0 ) Return
      Call Hcopy( id2,idnew,'BINOPER '//title )
C
c 3. Do operation
c
      Do J = 1, ny
        Do I = 1, Nx
          Call Hijxy ( id2, i, j, xlow, ylow )
          Call Hijxy ( id2, i+1, j+1, xhigh, yhigh )
          xbin = (xlow + xhigh)/2.0
          ybin = (ylow + yhigh)/2.0
          xwid = xhigh - xlow
          ywid = yhigh - ylow
          z = Hij( id2, i, j )
          Call HLbinoper_call2
     &         ( %VAL(Address), i, j, xbin, ybin, xwid, ywid, z )
          Rmem(loc+nx*(j-1)+i-1) = z
        End Do
      End Do
C
      Call HPAK ( idnew, Rmem(Loc) )
C
      Call DIS45_fixentry( idnew )
C
C 4. Free working buffer
C
      Call DIS45_free ( Idbuf )
C
C 5. Unlink dynamic load module
C
      Call DIS45_dynclose( address )
      Return
      End
C
C
      Subroutine HLbinoper_call2( Func, ix, iy, x, y, xw, yw, z )
c
      Implicit         NONE
c
c Arg
      External  Func
      Integer * 4  ix, iy
      Real * 4  x, y, xw, yw, z
c begin
      Call Func ( ix, iy, x, y, xw, yw, z )
C
      Return
      End
C
C

