C
C File: HLfitus.for
C Description: fit with user function
C Author: A.Shirahashi
C Date: 2-Jun-1992
C Original: HLfitga.for by T.Takahashi
C
C History:  5-Jun-1992  compute C.L. using NAG library
C          30-Jun-1992  static link if DIS45_dynlink returns 0
C           1-Jun-1992  fix a wrong call to static linked UFUNC
C           6-Jun-1992  now DIS45_dynlink returns staic address for UNIX
C                       version, thus make use of it.
C           6-Nov-1992  use PROB (CERNLIB) instaed of G01BCE (NAG)
C
C          11-Nov-1993  Call DIS45_DynClose
C
C This module is machine dependent because it uses %VAL.
C
      Subroutine  HLfitus(error)
C
      Implicit None
c output
      Integer error
C constant
      Integer  MAXPAR
      Parameter ( MAXPAR = 8 )
C common (for parameter passing)
      Double Precision  P
      Common /COMP/ P(MAXPAR)
C common (for dynamic linking)
      Integer  address
      Common /HLfitus_link/ address
C local
      Integer  nx
      Real  xmin, xmax
      Integer  here, id, dim
      Integer ic, np, i
      Real  sigma(MAXPAR), chi2
      Character * 80  title
      Integer  length
      Integer  ivx, ivy, ivdy
C     ...for MNstat
C     Real  fmin, fedm, errdef
C     Integer npari, nparx, istat
      Integer  ndf
      Real  cl
C function
      Integer  Lenrd, DIS45_needhist, DIS45_dynlink
      Real  PROB
      Double Precision  STFUNC, DYFUNC
      External  STFUNC, DYFUNC
C begin
C     require 1 dim (1) or N-tuples (3) histogram
      If (DIS45_needhist(4,error).le.0) Return
C
      Call DIS45_thishist(here, id, dim)
      Call DIS45_hinfo1(id,title,nx,xmin,xmax)
C
      Call HLfit_option(ic)
C
      Call IntrdL('?number of parameters',np,1,MAXPAR)
C
      If( nx.le.np ) Then
        error = 24 ! too small data points
        Return
      End If
C
      address = DIS45_dynlink('UFUNC','UFUNC')
      If( address .eq. 0 ) Then
C       ... static link
C       Call Hfits(id,STFUNC,np,P,chi2,ic,sigma)
        error = 28 ! can't load DYNLINK module
        Return
      End If
C
C     ... dynamic link
      If( dim.eq.1 ) Then
        Call HfitS(id,DYFUNC,np,P,chi2,ic,sigma)
      Else If( dim.eq.3 ) Then
        Call HLfitNT_askvar(id,ic,ivx,ivy,ivdy)
        Call HLfitNT(id,ivx,ivy,ivdy,DYFUNC,np,P,chi2,ic,sigma)
      Else
        Print *, 'HLfitus: can''t happen: dim = ', dim
        Return
      End If
C
      Call DIS45_aux_putinfo(id,'FUNC',4)
C
C     ...check fitting status
C     Call MNstat( fmin,fedm,errdef,npari,nparx,istat )
C     Print *, 'MNstat returns fmin=',fmin
C
C     ...compute C.L.
C     ...Hfits returns not chi2/ndf but chi2 itself.
      ndf = nx - np
      cl = PROB(chi2,ndf)
C
C print out results
      length = Lenrd(title)
      Write(6,1000) ID,title(1:length)
 1000 Format(5X,I5,1X,A)
C
      Write(6,1100) np, chi2/Float(nx-np), cl
 1100 Format(5X,
     &  '#param = ',I2,'  chi**2/ndf = ', G12.5, '  C.L. = ',F8.5)
C
      Do i = 1, np
        Write(6,1200) i, P(i), sigma(i)
 1200   Format(5X,'p(',I2,') = ',G12.5,' +- ',G12.5)
      End Do
C
C ... unlink shared object
      Call DIS45_dynclose(address)
C
      Return
      End
C
C
      Double Precision Function STFUNC( X )
C
      Implicit None
C input
      Real  X
C constant
      Integer  MAXPAR
      Parameter ( MAXPAR = 8 )
C common (for parameter passing)
      Double Precision  P
      Common /COMP/ P(MAXPAR)
C function
      Double Precision  HLfitus_call
      External  UFUNC
C begin
      STFUNC = HLfitus_call( X, P, UFUNC )
      Return
      End
C
C
      Double Precision Function DYFUNC( X )
C
      Implicit None
C input
      Real  X
C constant
      Integer  MAXPAR
      Parameter ( MAXPAR = 8 )
C common (for parameter passing)
      Double Precision  P
      Common /COMP/ P(MAXPAR)
C common (for dynamic linking)
      Integer  address
      Common /HLfitus_link/ address
C function
      Double Precision  HLfitus_call      
C begin
C
C     ... for dynamic link (VMS specific)
      DYFUNC = HLfitus_call( X, P, %VAL(address) )
      Return
      End
C
C
      Double Precision Function HLfitus_call( X, P, FUNC )
C
      Implicit None
C input
      Real  X
      Double Precision P(*), FUNC
      External FUNC
C begin
      HLfitus_call = FUNC( X, P )
      Return
      End
C      
