C  
C File: HLscale.for
C Description: various scaling commands
C Author: A.Shirahahsi
C Date: 14-May-1992
C Original: HLscal (FACOM version)
C
C History: 14-Jul-1992, opt 'B' valid for non-equiv. hist
C           3-Aug-1992, default value for f
C          24-Aug-1992, HLscales (SCALE)
C          21-Nov-1992, Add Option 'C' - eqv. to SCAC in FACOM-DIS
C                       by R. Itoh
C           6-Jan-1993, D(W): 1/Bin (integrate with bin width) by T.T.
C
C  SCALE, HSCAL, BSCAL, NSCAL
C
      Subroutine HLscales(error)
C
      Implicit None 
C output
      Integer error
C local
      Real  factor
      Character * 16  opt
      Integer  Lopt
C funcion
      Integer  Lenrd, DIS45_needhist
C data
      Data factor /1.0/
C begin
      If( DIS45_needhist(0,error).le.0 ) Return
C
      Call FltrdX( 
     &  'factor or (B:/bin,C:histo,H:/histo,N:entry,W:*bin,U:user)',
     &  factor,opt )
      Lopt = Lenrd(opt)
      Call Strupc(Lopt,opt)
      If( opt(1:1).eq.'B' ) Then
        Call HLscale('B',error)
      Else If( opt(1:1).eq.'H' ) Then
        Call HLscale('H',error)
      Else If( opt(1:1).eq.'N' ) Then
        Call HLscale('N',error)
      Else If ( opt(1:1).eq.'C' ) Then
        Call HLscale('C',error)
      Else If ( opt(1:1).eq.'D' .or. opt(1:1).eq.'W' ) Then
        Call HLscale('W',error)
      Else If( Index('0123456789+-.',opt(1:1)).gt.0 ) Then
        Call Ugetrd(opt,Lopt)
        Call HLscale('S',error)
      Else
        error = 23 ! illegal option
      End If
C
      Return
      End
C
C
      Subroutine HLscale( opt,error )
C scaling commands
      Implicit None
C common
      Include 'pawc.inc'
C input
      Character  opt
C output
      Integer  error
C local
      Character * 80  title
      Integer  here, id, idscal, dim
      Integer  nx, ny, nentry
      Integer  i, j
      Integer  size, idbuf, loc
      Real  xmin, xmax, ymin, ymax, low, high
      Real  s, f, sme
C
C function
      Integer  DIS45_needhist, DIS45_hasbar
      Integer  DIS45_hexist, DIS45_malloc
      Real  Hi, Hij, Hie, Hsum
C data
      Save  f
      Data f / 1.0 /
C
C begin
      If( DIS45_needhist(0,error).le.0 ) Return
      Call DIS45_thishist(here, id, dim)
C
      If( dim.eq.1 ) Then
        Call DIS45_hinfo1(id,title,nx,xmin,xmax)             
        size = nx * 2
      Else If( dim.eq.2 ) Then
        Call DIS45_hinfo2(id,title,nx,xmin,xmax,ny,ymin,ymax)
        size = nx * ny
      End If
C
      If( DIS45_malloc(size,idbuf,loc,error).le.0 ) Return
C
      If( opt.eq.' ' .or. opt.eq.'S' ) Then
        Call Fltrd('scaling factor',f)
      Else If( opt.eq.'H' ) Then
        idscal = id
        Call Intrd('scaling histogram',idscal)
        If( DIS45_hexist(idscal,error).le.0 ) Return
        s = Hsum( idscal )
        If( s.eq.0.0 ) Then
          error = 8 ! histogram emtpty
          Return
        End If
        f = 1.0/s
      Else If( opt.eq.'C' ) Then
        idscal = id
        Call Intrd('scaling histogram',idscal)
        If( DIS45_hexist(idscal,error).le.0 ) Return
        s = Hsum( idscal )
        If( s.eq.0.0 ) Then
          error = 8 ! histogram emtpty
          Return
        End If
        sme = Hsum( id )
        f = s / sme
      Else If( opt.eq.'B' .or. opt.eq.'W' ) Then
        If( dim.eq.2 ) Then
          error = 7 ! command not valie for 2-dim
          Return
        End If
      Else If( opt.eq.'N' ) Then
        s = Hsum( id )
        nentry = Int(s)
        Call Intrd('total entries',nentry)
        If( s.eq.0.0 ) Then
          error = 8 ! histogram empty
          Return
        End If
        f = Float(nentry)/s
      Else
        Print *, 'HLscale: no such option: ', opt
      End If
C
      Call DIS45_locate(idbuf,loc)
      If( dim.eq.1 ) Then     
        Do i = 0, nx-1
          If( opt.eq.'B' ) Then
            Call Hix(id,i+1,low)
            Call Hix(id,i+2,high)
            f = 1.0 / (high - low)
          Else if( opt.eq.'W' ) Then
            Call Hix(id,i+1,low)
            Call Hix(id,i+2,high)
            f = high - low
          End If
          Rmem(loc+i) = Hi(id,i+1) * f
          Rmem(loc+nx+i) = Hie(id,i+1) * f
        End Do
      Else If( dim.eq.2 ) Then         
        Do j = 0, ny-1
          Do i = 0, nx-1
            Rmem(loc+nx*j+i) = Hij(id,i+1,j+1) * f
          End Do
        End Do
      End If
C
      Call Hpak(id,Rmem(loc))
      If( dim.eq.1 .and. DIS45_hasbar(id).ne.0 ) Then
        Call Hpake(id,Rmem(loc+nx))
      End If
      Call DIS45_free(idbuf)
      Call DIS45_fixentry(id)
C
      Return
      End
